/**
 * Copyright (c) 2006 - 2008 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.uv.Responder;
import com.smaxe.uv.client.rtmp.INetConnection;

import java.io.FileInputStream;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Map;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

/**
 * <code>ExRtmpCreateNetConnection</code> - {@link NetConnection} usage example.
 * <p> Note:
 * <br> - The example shows how to create, configure, set callback object and connect to the server, invoke remote method.
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmp-client" target="_blank">JUV RTMP Client</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmfp-client" target="_blank">JUV RTMFP/RTMP Client</a>
 */
public final class ExRtmpCreateNetConnection extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception 
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        
        // Android-specific:
        // - please add permission to the AndroidManifest.xml
        // <uses-permission android:name="android.permission.INTERNET" />
        // - please use separate thread to connect to the server (not UI thread) : 
        // NetConnection#connect() connects to the remote server in the invocation thread,
        // so it causes NetworkOnMainThreadException on 4.0 (http://developer.android.com/reference/android/os/NetworkOnMainThreadException.html)
        com.smaxe.uv.client.rtmp.License.setKey("SET-YOUR-KEY");
        
        // RTMP example
        start(new com.smaxe.uv.client.rtmp.NetConnection(), "rtmp://184.72.239.149/vod/mp4:BigBuckBunny_175k.mov");
        
        // RTMPS example
        // start(new com.smaxe.uv.client.rtmp.NetConnection(), "rtmps://184.72.239.149/vod/mp4:BigBuckBunny_175k.mov");
        
        // RTMFP example
        // Note: This classes are available in the JUV RTMFP/RTMP Client library (juv-rtmfp-client-*.jar)
        // com.smaxe.uv.client.rtmfp.License.setKey("SET-YOUR-KEY");
        // 
        // start(new com.smaxe.uv.client.rtmfp.NetConnection(), "rtmfp://localhost:1935/live");
    }
    
    /**
     * Starts the example.
     * 
     * @param connection
     * @param url connection url
     * @throws Exception if an exception occurred
     */
    public static void start(final INetConnection connection, final String url) throws Exception
    {
        // configure NetConnection
        connection.configuration().put(INetConnection.Configuration.RTMPS_SSL_CONTEXT, createSSLContext("ssl.keystore", "your password"));
        
        // set client object to accept incoming server-to-client invocations
        connection.client(new ClientAPI());
        
        connection.addEventListener(new NetConnectionListener());
        
        connection.connect(url);
        
        Thread.sleep(10 * 1000);
        
        connection.close();
    }
    
    /**
     * <code>ClientAPI</code> - client API : server invokes methods of this class.
     */
    public static class ClientAPI extends Object
    {
        /**
         * Constructor.
         */
        public ClientAPI()
        {
        }
        
        /**
         * Test invocation.
         * 
         * @return result string
         */
        public String testInvocation()
        {
            System.out.println("ClientAPI#testInvocation()");
            
            // emulate time spent for method execution
            try
            {
                Thread.sleep(1000);
            }
            catch (Exception e) {/*ignore*/}
            
            return "ClientAPI#testInvocation() result";
        }
    }
    
    /**
     * <code>NetConnectionListener</code> - {@link INetConnection} listener implementation.
     */
    private static class NetConnectionListener extends INetConnection.ListenerAdapter
    {
        /**
         * Constructor.
         */
        public NetConnectionListener()
        {
        }
        
        @Override
        public void onAsyncError(final INetConnection source, final String message, final Exception e)
        {
            System.out.println("NetConnection#onAsyncError: " + message + " " + e);
        }
        
        @Override
        public void onIOError(final INetConnection source, final String message)
        {
            System.out.println("NetConnection#onIOError: " + message);
        }
        
        @Override
        public void onNetStatus(final INetConnection source, final Map<String, Object> info)
        {
            System.out.println("NetConnection#onNetStatus: " + info);
            
            final Object code = info.get("code");
            
            if (INetConnection.CONNECT_SUCCESS.equals(code))
            {
                source.call("testConnection", new Responder()
                {
                    public void onResult(final Object result)
                    {
                        System.out.println("Method testConnection result: " + result);
                    }
                    
                    public void onStatus(final Map<String, Object> status)
                    {
                        System.out.println("Method testConnection status: " + status);
                    }
                });
            }
        }
    }
    
    /**
     * Returns SSL Context.
     *
     * @param file certificate file (self-signed or any other)
     * @param password password
     * @return SSL Context
     */
    private static SSLContext createSSLContext(final String file, final String passwordx)
    {
        try
        {
            char[] pwd = passwordx.toCharArray();
            final InputStream is = new FileInputStream(file);
            
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509", "SunJSSE");
            KeyStore ks = KeyStore.getInstance("JKS");
            
            ks.load(is, pwd);
            is.close();
            
            kmf.init(ks, pwd);
            
            SSLContext context = SSLContext.getInstance("TLS");
            
            context.init(kmf.getKeyManagers(), new TrustManager[] {new X509TrustManager()
            {
                public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException
                {
                }
                public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException
                {
                }
                public X509Certificate[] getAcceptedIssuers()
                {
                    return null;
                }
            }}, null); 
            
            return context;
        }
        catch (Exception e)
        {
            return null;
        }
    } 
    
    /**
     * Constructor.
     */
    private ExRtmpCreateNetConnection()
    {
    }
}