/**
 * Copyright (c) 2006 - 2008 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.uv.ObjectEncoding;
import com.smaxe.uv.client.rtmp.INetConnection;
import com.smaxe.uv.client.rtmp.INetStream;
import com.smaxe.uv.client.rtmp.video.FlvVideo;
import com.smaxe.uv.client.rtmp.video.LogVideo;

import java.util.Map;

/**
 * <code>ExRtmpPlayStream</code> - {@link INetConnection} usage example.
 * <p> Note:
 * <br> - The example shows how to 'play' server stream. The played stream is saved to the local flv file.
 * 
 * @author Andrei Sochirca
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmp-client" target="_blank">JUV RTMP Client</a>
 * @see <a href="http://www.smaxe.com/product.jsf?id=juv-rtmfp-client" target="_blank">JUV RTMFP/RTMP Client</a>
 */
public final class ExRtmpPlayStream extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     * @throws Exception
     */
    public static void main(final String[] args) throws Exception
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        
        // Android-specific:
        // - please add permission to the AndroidManifest.xml : 
        // <uses-permission android:name="android.permission.INTERNET" />
        // - please use separate thread to connect to the server (not UI thread) : 
        // NetConnection#connect() connects to the remote server in the invocation thread,
        // so it causes NetworkOnMainThreadException on 4.0 (http://developer.android.com/reference/android/os/NetworkOnMainThreadException.html)
        
        com.smaxe.uv.client.rtmp.License.setKey("SET-YOUR-KEY");
        
        final ExRtmpPlayStream app = new ExRtmpPlayStream();
        
        // RTMP example
        app.start(new com.smaxe.uv.client.rtmp.NetConnection(), "rtmp://184.72.239.149/vod", "mp4:BigBuckBunny_175k.mov", false /*record*/);
        
        // RTMFP example
        // Note: This classes are available in the JUV RTMFP/RTMP Client library (juv-rtmfp-client-*.jar)
        // com.smaxe.uv.client.rtmfp.License.setKey("SET-YOUR-KEY");
        // app.start(new com.smaxe.uv.client.rtmfp.NetConnection(), "rtmfp://localhost:1935/live", "stream", false /*record*/);
    }
    
    /**
     * <code>NetConnectionListener</code> - {@link INetConnection} listener implementation.
     */
    public class NetConnectionListener extends INetConnection.ListenerAdapter
    {
        /**
         * Constructor.
         */
        public NetConnectionListener()
        {
        }
        
        @Override
        public void onAsyncError(final INetConnection source, final String message, final Exception e)
        {
            System.out.println("NetConnection#onAsyncError: " + message + " " + e);
        }
        
        @Override
        public void onIOError(final INetConnection source, final String message)
        {
            System.out.println("NetConnection#onIOError: " + message);
        }
        
        @Override
        public void onNetStatus(final INetConnection source, final Map<String, Object> info)
        {
            System.out.println("NetConnection#onNetStatus: " + info);
            
            final Object code = info.get("code");
            
            if (INetConnection.CONNECT_SUCCESS.equals(code))
            {
            }
            else
            {
                disconnected = true;
            }
        }
    }
    
    // fields
    private volatile boolean disconnected = false;
    
    /**
     * Constructor.
     */
    public ExRtmpPlayStream()
    {
    }
    
    /**
     * Starts the example.
     * 
     * @param connection
     * @param url
     * @param streamName stream name
     * @param record set <code>true</code> to record to the file
     * @throws Exception if an exception occurred
     */
    public void start(final INetConnection connection, final String url,
            final String streamName, final boolean record) throws Exception
    {
        connection.addEventListener(new NetConnectionListener());
        
        connection.objectEncoding(ObjectEncoding.AMF0);
        
        connection.connect(url);
        
        // wait till connected
        while (!connection.connected() && !disconnected)
        {
            try
            {
                Thread.sleep(100);
            }
            catch (Exception e) {/*ignore*/}
        }
        
        if (!disconnected)
        {
            final INetStream stream = connection.createNetStream();
            
            stream.addEventListener(new INetStream.ListenerAdapter()
            {
                @Override
                public void onNetStatus(final INetStream source, final Map<String, Object> info)
                {
                    System.out.println("NetStream#onNetStatus: " + info);
                }
            });
            
            // default buffer time is 100 (ms)
            // large buffer time let you download VOD stream faster
            // stream.bufferTime(3600000);
            
            stream.play(record ? new FlvVideo(streamName + ".flv") : new LogVideo(), streamName);
            
            Thread.sleep(60 * 1000);
            
            stream.close();
        }
        
        connection.close();
    }
}